@@ -1,86 +1,70 @@
-from __future__ import print_function
+#!/usr/bin/env python3
+
+import glob
+import logging
 import os
-DEBUG = False
-def to_bool(s): return s in [1,'True','TRUE','true','1','yes','Yes','Y','y','t']
-USE_CONFIG = "ACAPTURE_CONFIG" in os.environ
-if "DEBUG" in os.environ and to_bool(os.environ["DEBUG"]):
-    DEBUG = True
-    try: import __builtin__
-    except ImportError: import builtins as __builtin__
-    import inspect
-    def lpad(s,c): return s[0:c].ljust(c)
-    def rpad(s,c):
-        if len(s) > c: return s[len(s)-c:]
-        else: return s.rjust(c)
-    def print(*args, **kwargs):
-        s = inspect.stack()
-        __builtin__.print("\033[47m%s@%s(%s):\033[0m "%(rpad(s[1][1],20), lpad(str(s[1][3]),10), rpad(str(s[1][2]),4)),end="")
-        return __builtin__.print(*args, **kwargs)
-_g_open = open
 import platform
+import re
 import subprocess
 import sys
-import time
-import glob
-import numpy as np
-import multiprocessing
 import threading
-import re
+import time
+
+import numpy as np
+from easydict import EasyDict as edict
+
+_g_open = open
 try:
     import queue
 except ImportError:
     import Queue as queue
-from io import BytesIO
-import configparser
-import traceback
+logger = logging.getLogger(__name__)
 
 
-config = configparser.ConfigParser()
-ini_file_name = os.path.basename(__file__.split(".")[0] + ".ini")
-if os.path.exists(ini_file_name) and USE_CONFIG:
-    config.read(ini_file_name)
-else:
-    config["AsyncCamera"] = {"camera": 0, "fps": 60, "width": 1280, "height": 720, "format": "MJPG"}
-    config["AsyncVideo"] = {"file": "video.mp4", "loop": True, "frame_capture": False, "sound": True, "sound_volume": 0.3}
-    if USE_CONFIG: config.write(_g_open(ini_file_name, "w"))
+def to_bool(s):
+    return s in [1, 'True', 'TRUE', 'true', '1', 'yes', 'Yes', 'Y', 'y', 't']
 
 
-def get_config(): return {section: dict(config[section]) for section in config.sections()}
-
+config = edict()
+config.camera = edict({"camera": 0, "fps": 60, "width": 1280, "height": 720, "format": "MJPG"})
+config.video = edict({"file": "video.mp4", "loop": True, "frame_capture": False, "sound": True, "sound_volume": 0.3})
 
 try:
     from mss import mss
-except:
+except Exception as e:
+    print(e)
     print("Error: Does not exist screen capture library.")
     print("   > pip3 install mss")
 
-
 try:
     import cv2
-except:
+except Exception as e:
+    print(e)
     print("Error: Does not exist OpenCV library.")
     print("   > curl -sL http://install.aieater.com/setup_opencv | bash -")
     print("   or")
     print("   > pip3 install opencv-python")
     print("   or")
     print("   > apt install python3-opencv")
-    exit(9)
+    raise e
 
+oldstdout = sys.stdout
 try:
-    import os
-    import sys
     with _g_open(os.devnull, 'w') as f:
-        oldstdout = sys.stdout
         sys.stdout = f
-        from importlib import util as importlib_util 
+        from importlib import util as importlib_util
         if importlib_util.find_spec("pygame") is None:
+            sys.stdout = oldstdout
             print("Error: Does not exist sound mixer library.")
-            print("   > pip3 install pygame contextlib")
+            print("   > pip3 install pygame")
+        else:
+            import pygame
         sys.stdout = oldstdout
-except:
-    traceback.print_exc()
+except Exception as e:
+    sys.stdout = oldstdout
+    print(e)
     print("Error: Does not exist sound mixer library.")
-    print("   > pip3 install pygame contextlib")
+    print("   > pip3 install pygame")
 
 
 def which(program):
@@ -89,12 +73,16 @@ def which(program):
             cmd = subprocess.check_output("which " + program, shell=True)
             cmd = cmd.decode("utf8").strip()
             return cmd
-        except:
+        except Exception as e:
+            (e)
             return None
     else:
+
         def is_exe(fpath):
             return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
+
         fpath, fname = os.path.split(program)
+        (fname)
         if fpath:
             if is_exe(program):
                 return program
@@ -115,26 +103,34 @@ if FFMPEG is None:
 
 
 class BaseCapture(object):
-    def keyboard_listener(self, key, x, y): pass
+    def keyboard_listener(self, key, x, y):
+        pass
 
 
 class AsyncCamera(BaseCapture):
-    # format:YUYV/MJPG
     def __init__(self, fd=None, **kwargs):
         global FFMPEG
-        self.conf = config["AsyncCamera"]
+        self.conf = config.camera
 
-        for k in self.conf: setattr(self, k, self.conf[k])
-        for k in kwargs: setattr(self, k, kwargs[k])
-        def s_bool(s, k): setattr(s, k, to_bool(getattr(s, k)))
-        def s_int(s, k): setattr(s, k, int(getattr(s, k)))
-        def s_float(s, k): setattr(s, k, float(getattr(s, k)))
+        for k in self.conf:
+            setattr(self, k, self.conf[k])
+        for k in kwargs:
+            setattr(self, k, kwargs[k])
+
+        def s_bool(s, k):
+            setattr(s, k, to_bool(getattr(s, k)))
+
+        def s_int(s, k):
+            setattr(s, k, int(getattr(s, k)))
+
+        def s_float(s, k):
+            setattr(s, k, float(getattr(s, k)))
 
         s_int(self, "fps")
         s_int(self, "width")
         s_int(self, "height")
 
-        if fd == None:
+        if fd is None:
             if re.match(r"\d", self.camera) is not None:
                 fd = int(self.camera)
             else:
@@ -148,35 +144,40 @@ class AsyncCamera(BaseCapture):
         self.t.start()
         self.current = None
 
-    def destroy(self): self.q2.put(0)
+    def destroy(self):
+        self.q2.put(0)
 
-    def is_ended(self): return False
+    def is_ended(self):
+        return False
 
+    @staticmethod
     def func(q, q2, fd, opt):
         import cv2
         import time
-        v = cv2.VideoCapture(fd)
-        buffers = []
-        v.set(cv2.CAP_PROP_FOURCC, (ord(opt['format'][0]) << 0) + (ord(opt['format'][1]) << 8) + (ord(opt['format'][2]) << 16) + (ord(opt['format'][3]) << 24))
-        v.set(cv2.CAP_PROP_FPS, opt["fps"])
-        v.set(cv2.CAP_PROP_FRAME_WIDTH, opt["width"])
-        v.set(cv2.CAP_PROP_FRAME_HEIGHT, opt["height"])
-        cnt = 0
-        tm = time.time()
-        while v.isOpened():
-            stat, src = v.read()
-            if stat:
-                cnt += 1
-                if time.time() - tm > 1.0:
-                    if DEBUG: print("CameraFPS:", cnt, src.shape)
-                    cnt = 0
-                    tm = time.time()
-                if q.qsize() < 1:
-                    src = cv2.cvtColor(src, cv2.COLOR_BGR2RGB)
-                    q.put((time.time(), src))
-                if q2.qsize() > 0:
-                    print("Kill camera thread")
-                    return
+        try:
+            v = cv2.VideoCapture(fd)
+            # v.set(cv2.CAP_PROP_FOURCC, (ord(opt['format'][0]) << 0) + (ord(opt['format'][1]) << 8) + (ord(opt['format'][2]) << 16) + (ord(opt['format'][3]) << 24))
+            # v.set(cv2.CAP_PROP_FPS, opt["fps"])
+            # v.set(cv2.CAP_PROP_FRAME_WIDTH, opt["width"])
+            # v.set(cv2.CAP_PROP_FRAME_HEIGHT, opt["height"])
+            cnt = 0
+            tm = time.time()
+            while v.isOpened():
+                stat, src = v.read()
+                if stat:
+                    cnt += 1
+                    if time.time() - tm > 1.0:
+                        logger.debug(f"CameraFPS: {cnt} {src.shape}")
+                        cnt = 0
+                        tm = time.time()
+                    if q.empty():
+                        src = cv2.cvtColor(src, cv2.COLOR_BGR2RGB)
+                        q.put((time.time(), src))
+                    if q2.empty() is False:
+                        logger.info("Kill camera thread")
+                        return
+        except Exception as e:
+            print(e)
 
     def read(self):
         while self.current is None:
@@ -192,18 +193,26 @@ class AsyncCamera(BaseCapture):
 class AsyncVideo(BaseCapture):
     def __init__(self, fd=None, **kwargs):
         global FFMPEG
-        self.conf = config["AsyncVideo"]
+        self.conf = config.video
         self.lock = threading.Lock()
         self.frame_capture = False
         self.reset = 0
         self.key_queue = []
         self.queue = queue.Queue()
 
-        for k in self.conf: setattr(self, k, self.conf[k])
-        for k in kwargs: setattr(self, k, kwargs[k])
-        def s_bool(s, k): setattr(s, k, to_bool(getattr(s, k)))
-        def s_int(s, k): setattr(s, k, int(getattr(s, k)))
-        def s_float(s, k): setattr(s, k, float(getattr(s, k)))
+        for k in self.conf:
+            setattr(self, k, self.conf[k])
+        for k in kwargs:
+            setattr(self, k, kwargs[k])
+
+        def s_bool(s, k):
+            setattr(s, k, to_bool(getattr(s, k)))
+
+        def s_int(s, k):
+            setattr(s, k, int(getattr(s, k)))
+
+        def s_float(s, k):
+            setattr(s, k, float(getattr(s, k)))
 
         s_bool(self, "loop")
         s_bool(self, "frame_capture")
@@ -217,8 +226,10 @@ class AsyncVideo(BaseCapture):
         v = cv2.VideoCapture(fd, cv2.CAP_FFMPEG)
         self.start_time = 0
         self.offset = 0
-        self.seq = v.get(cv2.CAP_PROP_FRAME_COUNT)
+        self.total = self.seq = v.get(cv2.CAP_PROP_FRAME_COUNT)
         self.fps = v.get(cv2.CAP_PROP_FPS)
+        self.width = v.get(cv2.CAP_PROP_FRAME_WIDTH)
+        self.height = v.get(cv2.CAP_PROP_FRAME_HEIGHT)
         sound = None
         sound_fd = None
         if FFMPEG is not None and self.frame_capture is False and self.sound and self.sound_volume > 0:
@@ -233,7 +244,9 @@ class AsyncVideo(BaseCapture):
                     sound_fd = _g_open(sound, "rb")
                     pygame.mixer.init()
                     pygame.mixer.music.load(sound_fd)
-                except:
+                except Exception as e:
+                    logger.warning(e)
+                    logger.warning("Could not initialize sound.")
                     sound = None
                     sound_fd = None
 
@@ -250,6 +263,7 @@ class AsyncVideo(BaseCapture):
         self.current = (False, None)
         self.previous_frame = -1
         self.t = threading.Thread(target=self.func, args=())
+        # self.t = threading.Thread(target=self.func, args=())
         self.t.setName("AsyncVideo")
         self.t.setDaemon(True)
         self.t.start()
@@ -257,7 +271,8 @@ class AsyncVideo(BaseCapture):
     def keyboard_listener(self, key, x, y):
         self.key_queue.append(key)
 
-    def is_ended(self): return self.seq_is_ended
+    def is_ended(self):
+        return self.seq_is_ended
 
     def func(self):
         tm = time.time()
@@ -268,7 +283,7 @@ class AsyncVideo(BaseCapture):
             if time.time() - tm2 > 1.0:
                 if threading.main_thread().is_alive() is False:
                     self.destroy()
-                    if DEBUG: print("Leave from AsyncVideo thread")
+                    logger.info("Leave from AsyncVideo thread")
                     return
                 tm2 = time.time()
             if self.need_to_close: return
@@ -285,11 +300,13 @@ class AsyncVideo(BaseCapture):
                     cnt += 1
                     if time.time() - tm > 1.0:
                         tm = time.time()
-                        if DEBUG: print("VideoFPS:", cnt)
+                        logger.info(f"\033[0K\rVideoFPS: {cnt}\033[1A")
                         cnt = 0
                 else:
                     if self.loop:
                         self.v.set(cv2.CAP_PROP_POS_FRAMES, 0)
+                    else:
+                        self.seq_is_ended = True
             #############################################################################
             # Player
             else:
@@ -304,7 +321,7 @@ class AsyncVideo(BaseCapture):
                         continue
                     self.framecount += 1
                     if current_frame - self.framecount > 10 or self.reset:
-                        #print("SET",current_frame,self.framecount,self.reset)
+                        # logger.debug("SET",current_frame,self.framecount,self.reset)
                         self.v.set(cv2.CAP_PROP_POS_FRAMES, current_frame)
                         self.reset = 0
                         self.framecount = current_frame
@@ -317,13 +334,13 @@ class AsyncVideo(BaseCapture):
                     cnt += 1
                     if time.time() - tm > 1.0:
                         tm = time.time()
-                        if DEBUG: print("VideoFPS:", cnt)
+                        logger.info(f"\033[0KVideoFPS: {cnt}\033[1A")
                         cnt = 0
-                # print(self.framecount,self.seq,self.f,self.offset,time.time()-self.start_time)
+                # logger.debug(self.framecount,self.seq,self.f,self.offset,time.time()-self.start_time)
                 if self.seq <= self.f:
-                    if DEBUG: print("End")
+                    logger.debug("End")
                     if self.loop:
-                        if DEBUG: print("Loop")
+                        logger.debug("Loop")
                         self.v.set(cv2.CAP_PROP_POS_FRAMES, 0)
                         self.start_time = 0
                         self.framecount = 0
@@ -344,16 +361,16 @@ class AsyncVideo(BaseCapture):
             self.sound_fd.close()
         try:
             self.v.release()
-            if DEBUG: print("Released-AsyncVideo")
-        except:
-            traceback.print_exc()
+            logger.info("Released-AsyncVideo")
+        except Exception as e:
+            logger.warning(e)
 
     def read(self):
         tm = time.time()
         #############################################################################
         # Extractor
         if self.frame_capture:
-            if self.queue.qsize() > 0:
+            if self.queue.empty() is False:
                 return self.queue.get()
             return (False, None)
 
@@ -363,7 +380,6 @@ class AsyncVideo(BaseCapture):
         lock.acquire()
 
         key = 0
-        reset = 0
         if len(self.key_queue) > 0:
             key = self.key_queue.pop(0)
         if key:
@@ -389,7 +405,6 @@ class AsyncVideo(BaseCapture):
                 if self.seq <= pf:
                     self.start_time = 0
                     self.offset = 0
-                #print(key&0xFF)
             else:
                 if (key & 0xFF) == ord(b'q'):
                     return
@@ -439,8 +454,11 @@ class ImgFileStub(BaseCapture):
         self.f = cv2.imread(fd, cv2.IMREAD_COLOR)
         self.f = cv2.cvtColor(self.f, cv2.COLOR_BGR2RGB)
 
-    def is_ended(self): return False
-    def destroy(self): pass
+    def is_ended(self):
+        return False
+
+    def destroy(self):
+        pass
 
     def read(self):
         return (True, self.f)
@@ -462,8 +480,11 @@ class DirImgFileStub(BaseCapture):
                 f = os.path.join(self.f, f)
                 self.flist += [f]
 
-    def is_ended(self): return len(self.flist) == 0
-    def destroy(self): pass
+    def is_ended(self):
+        return len(self.flist) == 0
+
+    def destroy(self):
+        pass
 
     def read(self):
         while len(self.flist) > 0:
@@ -476,9 +497,14 @@ class DirImgFileStub(BaseCapture):
 
 
 class ScreenCapture(BaseCapture):
-    def __init__(self): self.need_to_close = False
-    def is_ended(self): return False
-    def destroy(self): self.need_to_close = True
+    def __init__(self):
+        self.need_to_close = False
+
+    def is_ended(self):
+        return False
+
+    def destroy(self):
+        self.need_to_close = True
 
     def read(self):
         with mss() as sct:
@@ -494,7 +520,7 @@ def open(f, **kwargs):
     if type(f) == str:
         if re.match(r"\d", f) is not None: f = int(f)
         if f == "-1": f = -1
-    if isinstance(f, (int,)):
+    if isinstance(f, (int, )):
         if f == -1:
             return ScreenCapture()
         return AsyncCamera(f, **kwargs)
@@ -539,9 +565,18 @@ def extract_video2images(f, **kwargs):
     format = "jpg"
     if "format" in kwargs:
         format = kwargs["format"]
-    cmd = "%s -i \"%s\" -qscale:v %d  \"%s/image_%%05d.jpg\"" % (FFMPEG, f, quality, dr,)
+    cmd = "%s -i \"%s\" -qscale:v %d  \"%s/image_%%05d.jpg\"" % (
+        FFMPEG,
+        f,
+        quality,
+        dr,
+    )
     if format == "png":
-        cmd = "%s -i \"%s\" -vcodec png \"%s/image_%%05d.png\"" % (FFMPEG, f, dr,)
+        cmd = "%s -i \"%s\" -vcodec png \"%s/image_%%05d.png\"" % (
+            FFMPEG,
+            f,
+            dr,
+        )
     print(mkdir)
     subprocess.call(mkdir, shell=True)
     print(cmd)
@@ -557,9 +592,19 @@ def compress_images2video(f, **kwargs):
         format = kwargs["format"]
     if "fps" in kwargs:
         fps = kwargs["fps"]
-    cmd = "%s -framerate %s -i \"%s/image_%%05d.jpg\" -vcodec libx264 -pix_fmt yuv420p -r 60 \"%s.out.mp4\"" % (FFMPEG, fps, f, f,)
+    cmd = "%s -y -framerate %s -i \"%s/image_%%05d.jpg\" -vcodec libx264 -pix_fmt yuv420p -r 60 \"%s.out.mp4\"" % (
+        FFMPEG,
+        fps,
+        f,
+        f,
+    )
     if format == "png":
-        cmd = "%s -framerate %s -i \"%s/image_%%05d.png\" -vcodec libx264 -pix_fmt yuv420p -r 60 \"%s.out.mp4\"" % (FFMPEG, fps, f, f,)
+        cmd = "%s -y -framerate %s -i \"%s/image_%%05d.png\" -vcodec libx264 -pix_fmt yuv420p -r 60 \"%s.out.mp4\"" % (
+            FFMPEG,
+            fps,
+            f,
+            f,
+        )
     print(cmd)
     subprocess.call(cmd, shell=True)
 
@@ -567,7 +612,11 @@ def compress_images2video(f, **kwargs):
 def extract_video2audio(f):
     global FFMPEG
     f = os.path.abspath(f)
-    cmd = "%s -i \"%s\" -ab 192 -ar 44100 \"%s.out.mp3\"" % (FFMPEG, f, f,)
+    cmd = "%s -i \"%s\" -ab 192 -ar 44100 \"%s.out.mp3\"" % (
+        FFMPEG,
+        f,
+        f,
+    )
     print(cmd)
     subprocess.call(cmd, shell=True)
 
@@ -577,10 +626,17 @@ def join_audio_with_video(f, sf):
     f = os.path.abspath(f)
     sf = os.path.abspath(sf)
     print(f, sf)
-    cmd = "%s -i \"%s\" -i \"%s\"  -map 0:v -map 1:a -c copy -shortest \"%s.mkv\"" % (FFMPEG, f, sf, f,)
+    cmd = "%s -i \"%s\" -i \"%s\"  -map 0:v -map 1:a -c copy -shortest \"%s.mkv\"" % (
+        FFMPEG,
+        f,
+        sf,
+        f,
+    )
     print(cmd)
     subprocess.call(cmd, shell=True)
 
+# TODO
+
 
 def convert(f, func):
     f = os.path.abspath(f)
@@ -596,7 +652,10 @@ def convert(f, func):
             img = std_resize(img)
             imgs = func(img)
             for im in imgs:
-                fname = "%s/image_%05d.png" % (dr, count,)
+                fname = "%s/image_%05d.png" % (
+                    dr,
+                    count,
+                )
                 cv2.imwrite(fname, im)
                 print(count)
                 count += 1
@@ -604,10 +663,10 @@ def convert(f, func):
             break
     print("Done")
     print("Clear")
-    subprocess.call("rm -f \"%s.mp3\"" % (f,), shell=True)
-    subprocess.call("rm -f \"%s.out.mp3\"" % (f,), shell=True)
-    subprocess.call("rm -f \"%s.out.mp4\"" % (dr,), shell=True)
-    subprocess.call("rm -f \"%s.out.mp4.mkv\"" % (dr,), shell=True)
+    subprocess.call("rm -f \"%s.mp3\"" % (f, ), shell=True)
+    subprocess.call("rm -f \"%s.out.mp3\"" % (f, ), shell=True)
+    subprocess.call("rm -f \"%s.out.mp4\"" % (dr, ), shell=True)
+    subprocess.call("rm -f \"%s.out.mp4.mkv\"" % (dr, ), shell=True)
     print("Create an audio")
     toaudio(f)
     print("Archiving....")
@@ -625,13 +684,64 @@ def gamma(img, g):
     return img
 
 
-if __name__ == '__main__':
-    import acapture
-    import pyglview
-    import sys
-    
-    cap = acapture.open(sys.argv[1] if len(sys.argv) > 1 else os.path.join(os.path.expanduser('~'), "test.mp4"))
+def take_one_shot(frame=100, format="jpg"):
+    pass
 
+
+class VideoWriteStream:
+    def __init__(self, **kargs):
+        if "input_stream" in kargs:
+            input_stream = kargs["input_stream"]
+            if input_stream.loop is not False:
+                raise Exception("Input stream loop flag must be false")
+            if input_stream.frame_capture is not True:
+                raise Exception("Input stream frame_capture must be true")
+        else:
+            raise Exception("input_stream arg did not specify.")
+
+        if "output" not in kargs:
+            raise Exception("output arg did not specify.")
+        output = kargs["output"]
+        format = "X264"
+        # format = "XVID"
+        if "format" in kargs:
+            format = kargs["format"]
+        # http://www.fourcc.org/codecs.php
+        fourcc = cv2.VideoWriter_fourcc(*format)
+        out = cv2.VideoWriter(output, fourcc, input_stream.fps, (int(input_stream.width), int(input_stream.height)))
+        self.writer = out
+
+    def write(self, frame):
+        self.writer.write(cv2.cvtColor(frame, cv2.COLOR_RGB2BGR))
+
+    def close(self):
+        self.writer.release()
+
+
+def test_video_write_stream():
+    import pyglview
+    import argparse
+
+    fd = os.path.expanduser("~/test.mp4")
+    output = os.path.expanduser("~/output.mp4")
+    cap = open(fd, frame_capture=True, loop=False)
+    wstream = VideoWriteStream(output=output, input_stream=cap)
+
+    while True:
+        check, frame = cap.read()
+        if check:
+            wstream.write(frame)
+            pass
+        if cap.is_ended():
+            break
+    wstream.close()
+
+
+def test_std():
+    import pyglview
+    import argparse
+
+    cap = open(os.path.expanduser(sys.argv[1]))
     view = pyglview.Viewer(keyboard_listener=cap.keyboard_listener, fullscreen=False)
 
     def loop():
@@ -639,11 +749,13 @@ if __name__ == '__main__':
             check, frame = cap.read()
             if check:
                 view.set_image(frame)
-                # view.set_image(gamma(frame, 0.6))
-        except:
-            traceback.print_exc()
+        except Exception as e:
+            logger.error(e)
             exit(9)
-        pass
+
     view.set_loop(loop)
     view.start()
-    print("Main thread ended")
+
+
+if __name__ == '__main__':
+    test_video_write_stream()
